from weaviate.collections.classes.batch import (
    DeleteManyObject,
    DeleteManyReturn,
)
from weaviate.collections.classes.data import Error, RefError

__all__ = [
    "DeleteManyObject",
    "DeleteManyReturn",
    "Error",
    "RefError",
]
