# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/health_weaviate.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18v1/health_weaviate.proto\x12\x0bweaviate.v1\"-\n\x1aWeaviateHealthCheckRequest\x12\x0f\n\x07service\x18\x01 \x01(\t\"\xa1\x01\n\x1bWeaviateHealthCheckResponse\x12\x46\n\x06status\x18\x01 \x01(\x0e\x32\x36.weaviate.v1.WeaviateHealthCheckResponse.ServingStatus\":\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x42k\n#io.weaviate.client.grpc.protocol.v1B\x0eWeaviateHealthZ4github.com/weaviate/weaviate/grpc/generated;protocolb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'v1.health_weaviate_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#io.weaviate.client.grpc.protocol.v1B\016WeaviateHealthZ4github.com/weaviate/weaviate/grpc/generated;protocol'
  _globals['_WEAVIATEHEALTHCHECKREQUEST']._serialized_start=41
  _globals['_WEAVIATEHEALTHCHECKREQUEST']._serialized_end=86
  _globals['_WEAVIATEHEALTHCHECKRESPONSE']._serialized_start=89
  _globals['_WEAVIATEHEALTHCHECKRESPONSE']._serialized_end=250
  _globals['_WEAVIATEHEALTHCHECKRESPONSE_SERVINGSTATUS']._serialized_start=192
  _globals['_WEAVIATEHEALTHCHECKRESPONSE_SERVINGSTATUS']._serialized_end=250
# @@protoc_insertion_point(module_scope)
