# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: v1/file_replication.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'v1/file_replication.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19v1/file_replication.proto\x12\x0bweaviate.v1\"Z\n\x18PauseFileActivityRequest\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\x12\x16\n\x0eschema_version\x18\x03 \x01(\x04\"C\n\x19PauseFileActivityResponse\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\"C\n\x19ResumeFileActivityRequest\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\"D\n\x1aResumeFileActivityResponse\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\":\n\x10ListFilesRequest\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\"O\n\x11ListFilesResponse\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\x12\x12\n\nfile_names\x18\x03 \x03(\t\"S\n\x16GetFileMetadataRequest\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\x12\x11\n\tfile_name\x18\x03 \x01(\t\"f\n\x0c\x46ileMetadata\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\x12\x11\n\tfile_name\x18\x03 \x01(\t\x12\x0c\n\x04size\x18\x04 \x01(\x03\x12\r\n\x05\x63rc32\x18\x05 \x01(\r\"~\n\x0eGetFileRequest\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12\x12\n\nshard_name\x18\x02 \x01(\t\x12\x11\n\tfile_name\x18\x03 \x01(\t\x12\x31\n\x0b\x63ompression\x18\x04 \x01(\x0e\x32\x1c.weaviate.v1.CompressionType\"6\n\tFileChunk\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0b\n\x03\x65of\x18\x03 \x01(\x08*\x87\x01\n\x0f\x43ompressionType\x12 \n\x1c\x43OMPRESSION_TYPE_UNSPECIFIED\x10\x00\x12\x19\n\x15\x43OMPRESSION_TYPE_GZIP\x10\x01\x12\x19\n\x15\x43OMPRESSION_TYPE_ZLIB\x10\x02\x12\x1c\n\x18\x43OMPRESSION_TYPE_DEFLATE\x10\x03\x32\xca\x03\n\x16\x46ileReplicationService\x12\x62\n\x11PauseFileActivity\x12%.weaviate.v1.PauseFileActivityRequest\x1a&.weaviate.v1.PauseFileActivityResponse\x12\x65\n\x12ResumeFileActivity\x12&.weaviate.v1.ResumeFileActivityRequest\x1a\'.weaviate.v1.ResumeFileActivityResponse\x12J\n\tListFiles\x12\x1d.weaviate.v1.ListFilesRequest\x1a\x1e.weaviate.v1.ListFilesResponse\x12U\n\x0fGetFileMetadata\x12#.weaviate.v1.GetFileMetadataRequest\x1a\x19.weaviate.v1.FileMetadata(\x01\x30\x01\x12\x42\n\x07GetFile\x12\x1b.weaviate.v1.GetFileRequest\x1a\x16.weaviate.v1.FileChunk(\x01\x30\x01\x42j\n#io.weaviate.client.grpc.protocol.v1B\rWeaviateProtoZ4github.com/weaviate/weaviate/grpc/generated;protocolb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'v1.file_replication_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n#io.weaviate.client.grpc.protocol.v1B\rWeaviateProtoZ4github.com/weaviate/weaviate/grpc/generated;protocol'
  _globals['_COMPRESSIONTYPE']._serialized_start=857
  _globals['_COMPRESSIONTYPE']._serialized_end=992
  _globals['_PAUSEFILEACTIVITYREQUEST']._serialized_start=42
  _globals['_PAUSEFILEACTIVITYREQUEST']._serialized_end=132
  _globals['_PAUSEFILEACTIVITYRESPONSE']._serialized_start=134
  _globals['_PAUSEFILEACTIVITYRESPONSE']._serialized_end=201
  _globals['_RESUMEFILEACTIVITYREQUEST']._serialized_start=203
  _globals['_RESUMEFILEACTIVITYREQUEST']._serialized_end=270
  _globals['_RESUMEFILEACTIVITYRESPONSE']._serialized_start=272
  _globals['_RESUMEFILEACTIVITYRESPONSE']._serialized_end=340
  _globals['_LISTFILESREQUEST']._serialized_start=342
  _globals['_LISTFILESREQUEST']._serialized_end=400
  _globals['_LISTFILESRESPONSE']._serialized_start=402
  _globals['_LISTFILESRESPONSE']._serialized_end=481
  _globals['_GETFILEMETADATAREQUEST']._serialized_start=483
  _globals['_GETFILEMETADATAREQUEST']._serialized_end=566
  _globals['_FILEMETADATA']._serialized_start=568
  _globals['_FILEMETADATA']._serialized_end=670
  _globals['_GETFILEREQUEST']._serialized_start=672
  _globals['_GETFILEREQUEST']._serialized_end=798
  _globals['_FILECHUNK']._serialized_start=800
  _globals['_FILECHUNK']._serialized_end=854
  _globals['_FILEREPLICATIONSERVICE']._serialized_start=995
  _globals['_FILEREPLICATIONSERVICE']._serialized_end=1453
# @@protoc_insertion_point(module_scope)
